import { EnchantmentType, system, world } from "@minecraft/server";
import { AnimationBlock, RedstonePowerChecker } from "../extended_functions.js";

// Registra o componente customizado no início do mundo
world.beforeEvents.worldInitialize.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent("ext:breaker_block_component", {
        onTick: breaker_tick,
    });
});

function breaker_tick(event) {
    const { block } = event;
    if (block.isAir) return;

    // Verifica se há energia redstone
    const redstoneChecker = new RedstonePowerChecker(block);
    const isPowered = redstoneChecker.getMaxPower() > 0;

    // Captura o estado de ativação atual do bloco
    const isActivated = block.permutation.getState("ext:activate") || false;
    // Atualiza o estado com base na energia
    block.setPermutation(block.permutation.withState("ext:activate", isPowered));

    // Se o bloco não estava ativado e agora recebe energia, inicia a ação
    if (!isActivated && isPowered) {
        // Recupera a direção do bloco
        let blockDirection = block.permutation.getState("minecraft:facing_direction");
        if (!blockDirection) return; // Se não houver direção definida, encerra a função

        const dimension = world.getDimension(block.dimension.id);
        const direction = directionData[blockDirection];
        if (!direction) return; // Se a direção não for válida, sai

        // Calcula a nova posição à frente do bloco
        const newLocation = {
            x: block.location.x + direction.x,
            y: block.location.y + direction.y,
            z: block.location.z + direction.z
        };

        const blockInLocation = dimension.getBlock(newLocation);
        // Verifica se o bloco na nova posição faz parte da lista de bloqueios
        const isBlockDenied =
            blocksDenyList.includes(blockInLocation.typeId) ||
            blockInLocation.getTags().some(tag => blockDenyTagList.includes(tag));

        if (!isBlockDenied) {
            // Inicia a animação do bloco usando o array de frames
            const animation = new AnimationBlock(block, [0, 1, 2, 3, 4, 5, 6, 5, 4, 3, 2, 1, 0], 1);
            animation.start();

            // Cria um intervalo para checar a animação a cada tick
            let animationRun = system.runInterval(() => {
                let animationActualFrame = animation.getCurrentFrame();
                if (animationActualFrame === 6) {
                    // Aplica dano e knockback às entidades próximas na nova localização
                    dimension.getEntitiesAtBlockLocation(newLocation).forEach(entityFront => {
                        try {
                            const impulseVector = { x: direction.x, y: direction.y, z: direction.z };
                            entityFront.applyDamage(30);
                            // Aplica o knockback com força horizontal fixa (1) e força vertical definida por impulseVector.y
                            entityFront.applyKnockback(impulseVector.x, impulseVector.z, 3, impulseVector.y);
                        } catch (error) {
                            // Em caso de erro, pode-se registrar em log se necessário
                        }
                    });

                    // Remove o bloco na posição afetada
                    dimension.runCommand(`setblock ${newLocation.x} ${newLocation.y} ${newLocation.z} air destroy`);
                    system.clearRun(animationRun);
                }
            }, 1); // Intervalo de 1 tick
        }
    }
}

const blockDenyTagList = [
    "ext:breaker_block_copper",
    "not_feature_replaceable"
];

const blocksDenyList = [
    "ext:breaker_block",
    "minecraft:obsidian",
    "minecraft:crying_obsidian",
    "minecraft:reinforced_deepslate",
    "minecraft:deny",
    "minecraft:allow",
    "minecraft:border_block",
    "minecraft:command_block",
    "minecraft:chain_command_block",
    "minecraft:repeating_command_block",
    "minecraft:ancient_debris"
];

const directionData = {
    "up": { x: 0, y: -1, z: 0 },
    "down": { x: 0, y: 1, z: 0 },
    "east": { x: -1, y: 0, z: 0 },
    "west": { x: 1, y: 0, z: 0 },
    "north": { x: 0, y: 0, z: 1 },
    "south": { x: 0, y: 0, z: -1 }
};
