import { EnchantmentType, system, world } from "@minecraft/server"
import { VFX_particle, fetchDynamicData, randomNumber, updateDynamicData, writeDynamicData, RedstonePowerChecker, isAllowedBlock } from "../extended_functions.js"

world.beforeEvents.worldInitialize.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent("ext:propellant_component", {
        onTick: propellant_tick,
    })
})

var animationFrame = 0;
var animationQuantityFrames = 11;

var maxStrength = 0.25 // Aplicado no jogador (Valor Original: 0.25 )
var multiplierFix = 0.02 // Aplicado nos Itens e outras entidades (Valor Original: 0.008 )
var range = 7; // Distância do Propulsor (NÃO modifique, pode afetar o desempenho) (Valor Original: 7 )

function propellant_tick(event) {
    const { block } = event;

    if (!new RedstonePowerChecker(block).getMaxPower()) {
        block.setPermutation(block.permutation.withState("ext:activate", false));
        return;
    }

    const blockLocation = block.bottomCenter();
    const dimension = world.getDimension(block.dimension.id);
    const blockDirection = block.permutation.getState("minecraft:facing_direction");
    const direction = directionData[blockDirection];

    if (!direction) return;

    let FireInWay = false;

    for (let i = 1; i <= range; i++) {
        const currentPos = {
            x: blockLocation.x + direction.x * i,
            y: blockLocation.y + direction.y * i,
            z: blockLocation.z + direction.z * i
        };

        const blockInFront = dimension.getBlock(currentPos);

        // Verifica se há fogo e netherrack no caminho
        if (blockInFront.typeId === "minecraft:fire" && dimension.getBlock({ x: currentPos.x, y: currentPos.y - 1, z: currentPos.z }).typeId === "minecraft:netherrack") {
            FireInWay = true;
        }

        block.setPermutation(block.permutation.withState("ext:animation_frame", animationFrame));

        const blockData = blocksData[blockInFront.typeId];
        if (blockData && ((FireInWay && blockData.withFire) || (!FireInWay && !blockData.withFire))) {
            updateDynamicData(`propellant=${JSON.stringify(blockInFront.location)}`, {
                location: blockInFront.location,
                propellant_location: block.location,
                blockID: blockInFront.typeId,
                dimensionID: blockInFront.dimension.id,
                age: randomNumber(blockData.age.min, blockData.age.max),
            });
        };

        if (!blockInFront.isAir && !isAllowedBlock(blockInFront)) {
            break;
        }

        const particleType = FireInWay ? "minecraft:basic_flame_particle" : "ext:propellant_particle";

        VFX_particle({ x: currentPos.x - 0.5, y: currentPos.y, z: currentPos.z - 0.5 }, block, particleType, 0.45, 0.02, 0.2);
        block.setPermutation(block.permutation.withState("ext:activate", true))

        // Loop para ajustar a coordenada Y para detecção de entidades
        // Dentro do loop para detecção de entidades:
        // Dentro do loop para detecção de entidades:
        for (let yFix = -0.5; yFix < 0.5; yFix += 0.25) {
            const currentPosEntities = {
                x: currentPos.x,
                y: currentPos.y + yFix,
                z: currentPos.z
            };

            dimension.getEntitiesAtBlockLocation(currentPosEntities)?.forEach(entity => {
                const directionX = (blockDirection === "east" || blockDirection === "west") ? direction.x : 0;
                const directionZ = (blockDirection === "north" || blockDirection === "south") ? direction.z : 0;
                const applyEffect = entitiesDeny[entity.typeId];

                if (FireInWay) {
                    entity?.setOnFire(5);
                }

                // Se a entidade estiver agachada, não aplicamos impulso horizontal.
                // Calcula-se apenas o impulso vertical e, se for para cima, ajusta para descer lentamente.
                if (entity.isSneaking) {
                    const knockbackWeight = 0.8;
                    const currentVelocityWeight = 1 - knockbackWeight;
                    const gravity = -0.98 / 16; // Constante de gravidade
                    const entityVelocity = entity.getVelocity();
                    let knockbackY = direction.verticalS * knockbackWeight + entityVelocity.y * currentVelocityWeight + gravity;

                    if (knockbackY > 0) {
                        knockbackY = -0.035; // Ajuste para uma descida suave
                    }
                    entity?.applyKnockback(0, 0, 0, knockbackY);
                    return;
                }

                if (applyEffect === false) return;

                if (applyEffect === true) {
                    entity?.applyImpulse({
                        x: directionX * multiplierFix,
                        y: direction.verticalS * 0.035,
                        z: directionZ * multiplierFix
                    });
                } else {
                    const knockbackWeight = 0.8;
                    const currentVelocityWeight = 1 - knockbackWeight;
                    const gravity = -0.98 / 16;
                    const entityVelocity = entity.getVelocity();

                    let knockbackX = directionX * knockbackWeight + entityVelocity.x * currentVelocityWeight;
                    let knockbackZ = directionZ * knockbackWeight + entityVelocity.z * currentVelocityWeight;
                    let knockbackY = direction.verticalS * knockbackWeight + entityVelocity.y * currentVelocityWeight + gravity;

                    entity?.applyKnockback(
                        knockbackX,
                        knockbackZ,
                        direction.horizontalS * knockbackWeight,
                        knockbackY
                    );
                }
            });
        }

    }
}

system.runInterval(() => {
    const timer = fetchDynamicData("timerGlobal")?.timer;
    world.getDynamicPropertyIds().forEach(dynamicID => {
        if (!dynamicID.startsWith("propellant")) return;
        const dynamicData = fetchDynamicData(dynamicID);

        if (!dynamicData?.location) return;
        const dimension = world.getDimension(dynamicData.dimensionID);
        const blockInLocation = dimension.getBlock(dynamicData.location);

        if (dynamicData?.propellant_location) {
            const blockInPropellant = dimension.getBlock(dynamicData?.propellant_location)
            if (!blockInPropellant.isAir) {
                const blockPropellantPermutation = blockInPropellant.permutation.getState("ext:activate")
                if (!blockPropellantPermutation) return;
            } else {
                world.setDynamicProperty(dynamicID, null);
                return;
            }
        }

        if (!blockInLocation) return;

        if (timer < 20) {
            if (blockInLocation.isAir) {
                world.setDynamicProperty(dynamicID, null);
            }
            return;
        }

        if (dynamicData.age > 0) {
            writeDynamicData(dynamicID, { ...dynamicData, age: dynamicData.age - 1 });
            return;
        }

        const blockData = blocksData[dynamicData.blockID];
        if (blockData.mode === "permutation") {
            blockInLocation.setPermutation(
                blockInLocation.permutation.withState(blockData.permutation.name, blockData.permutation.value)
            );
        } else if (blockData.mode === "setblock") {
            // Considerando que dynamicID seja uma string serializada (por isso JSON.parse)
            dimension.setBlockType(dynamicData.location, blockData.blockID);
            world.setDynamicProperty(dynamicID, null);
        }
    });
});

system.runInterval(() => {
    if (animationFrame < animationQuantityFrames) {
        animationFrame++
        return;
    }
    animationFrame = 0
}, 1.5)

const blocksFrontAllow = [
    "minecraft:fire"
];

const directionData = {
    "east": { horizontalS: maxStrength, verticalS: 0, x: -1, y: 0, z: 0 },
    "west": { horizontalS: maxStrength, verticalS: 0, x: 1, y: 0, z: 0 },
    "north": { horizontalS: maxStrength, verticalS: 0, x: 0, y: 0, z: 1 },
    "south": { horizontalS: maxStrength, verticalS: 0, x: 0, y: 0, z: -1 },
    "up": { horizontalS: 0, verticalS: -maxStrength, x: 0, y: -1, z: 0 },
    "down": { horizontalS: 0, verticalS: maxStrength, x: 0, y: 1, z: 0 }
};

const entitiesDeny = {
    "minecraft:item": true,
    "minecraft:tnt_minecart": true,
    "minecraft:falling_block": false,
    "minecraft:arrow": true,
    "minecraft:snowball": true,
    "minecraft:ender_pearl": true,
    "minecraft:egg": true,
    "minecraft:minecart": true,
    "minecraft:xp_bottle": true,
    "minecraft:splash_potion": true,
    "minecraft:thrown_trident": true,
    "minecraft:fireworks_rocket": true,
    "minecraft:tnt": true,
    "minecraft:boat": true
};

const blocksData = {
    "minecraft:campfire": { age: { min: 3, max: 6 }, withFire: false, mode: "permutation", permutation: { name: "extinguished", value: true } },
    "minecraft:soul_campfire": { age: { min: 3, max: 6 }, withFire: false, mode: "permutation", permutation: { name: "extinguished", value: true } },
    "minecraft:fire": { age: { min: 3, max: 4 }, withFire: false, mode: "setblock", blockID: "minecraft:air" },

    "minecraft:sand": { age: { min: 5, max: 10 }, withFire: true, mode: "setblock", blockID: "minecraft:glass" },
    "minecraft:wet_sponge": { age: { min: 5, max: 10 }, withFire: true, mode: "setblock", blockID: "minecraft:sponge" },
    "minecraft:cobblestone": { age: { min: 8, max: 16 }, withFire: true, mode: "setblock", blockID: "minecraft:stone" },
    "minecraft:stone_bricks": { age: { min: 8, max: 16 }, withFire: true, mode: "setblock", blockID: "minecraft:cracked_stone_bricks" },
    "minecraft:ice": { age: { min: 2, max: 4 }, withFire: true, mode: "setblock", blockID: "minecraft:flowing_water" },
};