import { ItemStack, world, system } from "@minecraft/server";
import { VFX_particle, randomNumber } from "../extended_functions.js";

const defaultFurnaceTime = 1;
const stackableTimers = {}; // Armazena os timers para itens empilháveis

const getItemCooked_ID = {
  // Minérios crus para lingotes
  "minecraft:raw_copper": "minecraft:copper_ingot",
  "minecraft:raw_iron": "minecraft:iron_ingot",
  "minecraft:raw_gold": "minecraft:gold_ingot",
  "minecraft:gold_ore": "minecraft:gold_ingot",
  "minecraft:iron_ore": "minecraft:iron_ingot",
  "minecraft:copper_ore": "minecraft:copper_ingot",
  "minecraft:nether_gold_ore": "minecraft:gold_nugget",
  "minecraft:ancient_debris": "minecraft:netherite_scrap",

  // Minérios de deepslate para lingotes
  "minecraft:deepslate_copper_ore": "minecraft:copper_ingot",
  "minecraft:deepslate_iron_ore": "minecraft:iron_ingot",
  "minecraft:deepslate_gold_ore": "minecraft:gold_ingot",

  // Carnes cruas para assadas
  "minecraft:beef": "minecraft:cooked_beef",
  "minecraft:chicken": "minecraft:cooked_chicken",
  "minecraft:porkchop": "minecraft:cooked_porkchop",
  "minecraft:mutton": "minecraft:cooked_mutton",
  "minecraft:rabbit": "minecraft:cooked_rabbit",
  "minecraft:cod": "minecraft:cooked_cod",
  "minecraft:salmon": "minecraft:cooked_salmon",

  // Vegetais e plantas
  "minecraft:potato": "minecraft:baked_potato",
  "minecraft:kelp": "minecraft:dried_kelp",
  "minecraft:cactus": "minecraft:green_dye",
  "minecraft:sea_pickle": "minecraft:lime_dye",
  "minecraft:chorus_fruit": "minecraft:popped_chorus_fruit",

  // Blocos fundidos/cozidos
  "minecraft:clay": "minecraft:terracotta",
  "minecraft:clay_ball": "minecraft:brick",
  "minecraft:sand": "minecraft:glass",
  "minecraft:red_sand": "minecraft:glass",
  "minecraft:cobblestone": "minecraft:stone",
  "minecraft:stone": "minecraft:smooth_stone",
  "minecraft:netherrack": "minecraft:nether_brick",
  "minecraft:wet_sponge": "minecraft:sponge",
  "minecraft:cobbled_deepslate": "minecraft:deepslate",
  "minecraft:sandstone": "minecraft:smooth_sandstone",
  "minecraft:red_sandstone": "minecraft:smooth_red_sandstone",
  "minecraft:basalt": "minecraft:smooth_basalt",
  "minecraft:stone_bricks": "minecraft:cracked_stone_bricks",

  // Itens do Nether e outros
  "minecraft:quartz_ore": "minecraft:quartz",
  "minecraft:rotten_flesh": "ext:rotten_flesh_powder",

  // Madeiras para carvão (charcoal)
  "minecraft:oak_log": "minecraft:charcoal",
  "minecraft:spruce_log": "minecraft:charcoal",
  "minecraft:birch_log": "minecraft:charcoal",
  "minecraft:jungle_log": "minecraft:charcoal",
  "minecraft:acacia_log": "minecraft:charcoal",
  "minecraft:dark_oak_log": "minecraft:charcoal",

  "minecraft:potato": "minecraft:baked_potato",
};

world.beforeEvents.worldInitialize.subscribe(event => {
  event.blockComponentRegistry.registerCustomComponent("ext:reinforced_furnace", {
    onTick: reinforced_furnace_tick,
    beforeOnPlayerPlace: reinforced_furnace_place,
    onPlayerDestroy: reinforced_furnace_destroy,
  });
});

function reinforced_furnace_tick(event) {
  const { block } = event;
  const dimension = world.getDimension(block.dimension.id);
  const entity = dimension
    .getEntitiesAtBlockLocation(block.bottomCenter())
    .find(e => e.typeId === "ext:reinforced_furnace_storage");
  if (!entity) return;

  const inventoryComponent = entity.getComponent("minecraft:inventory");
  if (!inventoryComponent) return;

  let isBurning = false;

  function hasEmptySlot() {
    for (let i = 0; i < inventoryComponent.inventorySize; i++) {
      if (!inventoryComponent.container.getItem(i)) return true;
    }
    return false;
  }

  // Processa os slots do inventário
  for (let slotIndex = 0; slotIndex < inventoryComponent.inventorySize; slotIndex++) {
    let item = inventoryComponent.container.getItem(slotIndex);
    if (!item) continue;
    if (!getItemCooked_ID[item.typeId]) continue;

    isBurning = true;

    if (item.isStackable) {
      // Utiliza o armazenamento externo para itens empilháveis
      if (stackableTimers[slotIndex] === undefined) {
        stackableTimers[slotIndex] = defaultFurnaceTime;
      }
      if (stackableTimers[slotIndex] > 0) {
        stackableTimers[slotIndex]--;
      } else {
        if (!hasEmptySlot()) continue;
        // Processa o cozimento do item empilhável
        const cookedItemID = getItemCooked_ID[item.typeId];
        const cookedItem = new ItemStack(cookedItemID, 1);
        inventoryComponent.container.addItem(cookedItem);
        if (item.amount > 1) {
          item.amount--;
          stackableTimers[slotIndex] = defaultFurnaceTime;
          inventoryComponent.container.setItem(slotIndex, item);
        } else {
          inventoryComponent.container.setItem(slotIndex, null);
          delete stackableTimers[slotIndex];
        }
      }
    } else {
      // Para itens não empilháveis, usa propriedade dinâmica
      let timer = item.getDynamicProperty("timer");
      if (timer === undefined) {
        timer = defaultFurnaceTime;
        try {
          item.setDynamicProperty("timer", timer);
        } catch (error) {
          console.error(`Erro ao definir propriedade dinâmica no slot ${slotIndex}: ${error}`);
          continue;
        }
      }
      if (timer > 0) {
        timer--;
        try {
          item.setDynamicProperty("timer", timer);
        } catch (error) {
          console.error(`Erro ao atualizar propriedade dinâmica no slot ${slotIndex}: ${error}`);
        }
      } else {
        if (!hasEmptySlot()) continue;
        const cookedItemID = getItemCooked_ID[item.typeId];
        const cookedItem = new ItemStack(cookedItemID, 1);
        inventoryComponent.container.addItem(cookedItem);
        inventoryComponent.container.setItem(slotIndex, null);
      }
    }
    // Se desejar processar apenas 1 slot por tick, descomente o break abaixo.
    break;
  }

  block.setPermutation(block.permutation.withState("ext:burnable", isBurning));

  if (isBurning) {
    const currentTime = Date.now();
    if (currentTime - lastParticleTime >= particleCooldown) {
      VFX_particle(block.location, block, "minecraft:campfire_smoke_particle", 0.01, randomNumber(1, 5), 0.5);
      lastParticleTime = currentTime;
    }
  }
}

let lastParticleTime = 0;
const particleCooldown = 800;

function reinforced_furnace_destroy(event) {
  const { block } = event;
  const dimension = world.getDimension(block.dimension.id);
  const entity = dimension
    .getEntitiesAtBlockLocation(block.bottomCenter())
    .find(e => e.typeId === "ext:reinforced_furnace_storage");

  if (entity) {
    const inventoryComponent = entity.getComponent("minecraft:inventory");
    if (inventoryComponent) {
      for (let slotIndex = 0; slotIndex < inventoryComponent.inventorySize; slotIndex++) {
        const item = inventoryComponent.container.getItem(slotIndex);
        if (item) {
          dimension.spawnItem(new ItemStack(item.typeId, item.amount), block.bottomCenter());
        }
      }
    }
    entity.remove();
  }
}

system.runInterval(() => {
  world.getAllPlayers().forEach(player => {
    const dimension = world.getDimension(player.dimension.id);
    dimension.getEntities({ type: "ext:reinforced_furnace_storage" }).forEach(entity => {
      const blockInLocation = dimension.getBlock(entity.location)
      if (blockInLocation.typeId != "ext:reinforced_furnace") {
        if (entity) {
          const inventoryComponent = entity.getComponent("minecraft:inventory");
          if (inventoryComponent) {
            for (let slotIndex = 0; slotIndex < inventoryComponent.inventorySize; slotIndex++) {
              const item = inventoryComponent.container.getItem(slotIndex);
              if (item) {
                dimension.spawnItem(new ItemStack(item.typeId, item.amount), entity.location);
              }
            }
          }
          entity.remove();
        }
      }
    })
  })
}, 1)

function reinforced_furnace_place(event) {
  const dimension = world.getDimension(event.block.dimension.id);
  let entitySpawned = dimension.spawnEntity("ext:reinforced_furnace_storage", event.block.bottomCenter());
  entitySpawned.nameTag = "entity.reinforced_furnace_gui.name";
}
